from tgtg import TgtgClient
from slack_sdk.webhook import WebhookClient
import os
import json
import threading
from typing import List, Dict, Any
from dataclasses import dataclass
from abc import ABC, abstractmethod
from datetime import datetime
from rich.console import Console
from rich.table import Table
from rich import print as rprint


# ================ Configuration ================
@dataclass
class TGTGConfig:
    access_token: str
    refresh_token: str
    user_id: str
    cookie: str

@dataclass
class SlackConfig:
    webhook_url: str

@dataclass
class AppConfig:
    tgtg: TGTGConfig
    slack: SlackConfig
    check_interval: int = 60
    cache_file: str = 'panier.json'

# ================ Notifications ================
@dataclass
class NotificationPayload:
    title: str
    message: str
    items_available: int
    store_name: str

class NotificationService(ABC):
    @abstractmethod
    def send_notification(self, payload: NotificationPayload) -> None:
        pass

class SlackNotificationService(NotificationService):
    def __init__(self, webhook_url: str):
        self.webhook = WebhookClient(webhook_url)
    
    def send_notification(self, payload: NotificationPayload) -> None:
        message = f"[TGTG] - {payload.items_available} PANIER DISPO - {payload.store_name}"
        self.webhook.send(text=message)

# ================ Cache Service ================
@dataclass
class CacheEntry:
    store_id: str
    items_available: int

class CacheService:
    def __init__(self, cache_file: str):
        self.cache_file = cache_file
        self._ensure_cache_file()
    
    def _ensure_cache_file(self) -> None:
        if not os.path.exists(self.cache_file):
            self.save_cache([])
    
    def load_cache(self) -> List[Dict[str, Any]]:
        with open(self.cache_file, "r") as json_file:
            return json.load(json_file)
    
    def save_cache(self, data: List[Dict[str, Any]]) -> None:
        with open(self.cache_file, "w") as json_file:
            json.dump(data, json_file)

# ================ TGTG Monitor ================
class TGTGMonitor:
    def __init__(self, 
                 config: AppConfig, 
                 notification_service: NotificationService,
                 cache_service: CacheService):
        self.config = config
        self.notification_service = notification_service
        self.cache_service = cache_service
        self.console = Console()
        self.client = TgtgClient(
            access_token=config.tgtg.access_token,
            refresh_token=config.tgtg.refresh_token,
            user_id=config.tgtg.user_id,
            cookie=config.tgtg.cookie
        )
        
    def _get_all_favorites(self) -> List[Dict[str, Any]]:
        """Get all favorite items with pagination"""
        try:
            # First try with get_favorites() which should return all favorites
            items = self.client.get_favorites()
            if not items:  # Fallback to get_items() if get_favorites() returns empty
                items = self.client.get_items(favorites_only=True, page_size=300)
            return items
        except Exception as e:
            print(f"Error getting favorites: {e}")
            return []

    def _display_stores_status(self, items: List[Dict[str, Any]]) -> None:
        """Display stores status in a clean table format"""
        table = Table(title=f"\nTGTG Monitor Status - {datetime.now().strftime('%H:%M:%S')}")
        
        # Add columns
        table.add_column("Store Name", style="cyan")
        table.add_column("Available", style="green")
        table.add_column("Status", style="bold")
        
        # Sort items by availability (available first) and then by name
        sorted_items = sorted(
            items,
            key=lambda x: (-x["items_available"], x["display_name"])
        )

        total_monitored = len(items)
        total_available = sum(1 for item in items if item["items_available"] > 0)
        
        for item in sorted_items:
            name = item["display_name"]
            available = str(item["items_available"])
            status = "?? AVAILABLE" if item["items_available"] > 0 else "?? waiting"
            
            table.add_row(
                name[:50] + "..." if len(name) > 50 else name,
                available,
                status
            )

        # Clear console before printing new status
        os.system('cls' if os.name == 'nt' else 'clear')
        
        # Print summary and table
        self.console.print(f"\n[bold]Monitoring {total_monitored} stores | {total_available} currently available[/bold]")
        self.console.print(table)
        
    def _check_and_notify(self, item: Dict[str, Any], cached_stores: List[Dict[str, Any]]) -> bool:
        """Check if notification is needed and send if necessary"""
        store_id = item["store"]["store_id"]
        items_available = item["items_available"]
        
        # Check if notification was already sent
        for cached_store in cached_stores:
            if (str(cached_store["id"]) == str(store_id) and 
                items_available == cached_store["panier"]):
                return False
                
        if items_available > 0:
            self._send_notification(item)
            return True
        return False
    
    def _send_notification(self, item: Dict[str, Any]) -> None:
        """Send notification for available items"""
        payload = NotificationPayload(
            title="PANIER RESERVABLE",
            message=f"{item['items_available']} Panier disponible",
            items_available=item["items_available"],
            store_name=item["display_name"]
        )
        self.notification_service.send_notification(payload)
    
    def _update_cache(self, stores: List[Dict[str, Any]]) -> None:
        """Update cache with current store states"""
        cache_data = [
            {"id": store["store"]["store_id"], "panier": store["items_available"]}
            for store in stores
        ]
        self.cache_service.save_cache(cache_data)
    
    def check_stores(self) -> None:
        """Main method to check stores and handle notifications"""
        try:
            cached_stores = self.cache_service.load_cache()
            items = self._get_all_favorites()
            
            # Process notifications
            for item in items:
                self._check_and_notify(item, cached_stores)
            
            # Update display
            self._display_stores_status(items)
            
            # Update cache
            self._update_cache(items)
            
        except Exception as e:
            rprint(f"[red]Error checking stores: {e}[/red]")
        finally:
            self._schedule_next_check()
    
    def _schedule_next_check(self) -> None:
        """Schedule the next store check"""
        threading.Timer(self.config.check_interval, self.check_stores).start()
    
    def start(self) -> None:
        """Start the monitoring process"""
        print("Starting TGTG Monitor...")
        self.check_stores()

# ================ Main ================
def main():
    # Initialize configurations
    config = AppConfig(
        tgtg=TGTGConfig(
            access_token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzA3MjE0MTcsImlhdCI6MTczMDU0ODYxNywiaXNzIjoidGd0Z19zb3RlcmlhIiwidCI6IjNVRFRVLUc2U0lTNDZGQ1d6d3hNUFE6MDoxIiwic3ViIjoiMTg4NTYyMDAifQ.W4vP5dWREOY62Z64x0AdfaigPKQB9M09kZABQzGbHS0",
            refresh_token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NjIwODQ2MTcsImlhdCI6MTczMDU0ODYxNywiaXNzIjoidGd0Z19zb3RlcmlhIiwidCI6InZmYmNFeEIwUUdlUUl3dXllX29oZWc6MDowIiwic3ViIjoiMTg4NTYyMDAifQ.8WmnaO_q_dSoj8BJhNx2zTU3Lv7o_y-psujOBvPD-zw",
            user_id="18856200",
            cookie="datadome=yFdHErLIwUdhm6DsA6yJb87oOP52POMovhAoyYN2aTtt5qppTPe~OR2LhpfR3l~N5053A~_Nq8OSC5iE3cPZsZxv_im5qbDDqUNiFbQGLFT~G4WEsgNPqegYQGnx2BxI; Max-Age=5184000; Domain=.apptoogoodtogo.com; Path=/; Secure; SameSite=Lax"
        ),
        slack=SlackConfig(
            webhook_url="https://hooks.slack.com/services/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"
        )
    )
    
    # Initialize services
    notification_service = SlackNotificationService(config.slack.webhook_url)
    cache_service = CacheService(config.cache_file)
    
    # Initialize and start monitor
    monitor = TGTGMonitor(config, notification_service, cache_service)
    monitor.start()

if __name__ == "__main__":
    main()